
# q-expansion of newform 2960.2.a.b, downloaded from the LMFDB on 12 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 2960
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [-2], [1], [-3], [-3], [0], [5], [2], [6], [1], [1], [1], [-5], [-1], [-4], [-9], [-8], [5], [0], [2], [-4], [-8], [4], [10], [-9], [-18], [8], [-6], [1], [15], [4], [-16], [4], [-19], [-18], [12], [7], [19], [-16], [-13], [-4], [-10], [-1], [-14], [2], [8], [-13], [-5], [-3], [-10], [24], [-21], [-12], [6], [-2], [19], [-22], [18], [-12], [20], [-4], [25], [-26], [3], [6], [-9], [10], [-12], [-20], [-14], [-3], [12], [-7], [10], [16], [20], [-37], [-10], [-14], [-32], [36], [2], [-21], [18], [7], [-34], [-6], [-17], [-5], [4], [-21], [32], [-2], [-36], [6], [36], [24], [1], [-36], [30], [-9], [-12], [21], [-6], [15], [2], [-45], [44], [16], [5], [-10], [-23], [16], [-19], [13], [39], [-29], [4], [10], [-20], [39], [-50], [38], [-39], [19], [34], [-15], [-34], [-40], [37], [-27], [-1], [-30], [-2], [11], [-16], [45], [-18], [46], [14], [48], [12], [-24], [-19], [13], [4], [-38], [-33], [-44], [21], [23], [-33], [-35], [-21], [28], [-56], [40], [5], [-46], [10], [-55], [0], [52], [23], [13], [-39], [19], [60], [-7], [-6], [-10], [5], [7], [-56], [-62], [-54], [-2], [6], [17], [-36], [-38], [-8], [-6], [-33], [45], [58], [37], [11], [-8], [44], [58], [-28], [-6], [48], [30], [6], [2], [-20], [10], [54], [46], [-20], [-49], [-39], [51], [28], [-26], [28], [50], [-13], [-5], [-27], [-52], [58], [-27], [31], [20], [0], [48], [57], [-26], [40], [15], [-48], [-72], [-10], [-48], [-38], [-7], [-14], [11], [8], [-44], [-10], [34], [59], [62], [-44], [48], [-48], [31], [-44], [-64], [-25], [-14], [-21], [-22], [16], [-21], [-40], [-58], [58], [17], [40], [-50], [27], [-32], [46], [-21], [-47], [30], [-8], [46], [23], [66], [-5], [8], [-60], [-44], [-74], [-42], [-75], [10], [29], [-14], [-5], [18], [-50], [9], [4], [64], [-69], [-21], [-44], [20], [18], [11], [23], [40], [6], [47], [-12], [-60], [-27], [79], [-10], [-26], [-34], [42], [-18], [63], [-62], [0], [36], [38], [53], [17], [-26], [-30], [36], [40], [34], [-25], [-56], [32], [-56], [39], [24], [64], [48], [48], [-22], [-9], [11], [-60], [-2], [-37], [1], [68], [69], [18], [-6], [29], [4], [-82], [-60], [-20], [56], [74], [-48], [74], [78], [66], [29], [36], [-64], [2], [2], [-93], [-69], [-29], [-4], [-71], [4], [-44], [-16], [53], [38], [-42], [-84], [40], [-62], [-22], [-28], [-58], [81], [-72], [44], [-62], [-28], [46], [-21], [-20], [23], [7], [60], [-24], [64], [-54], [-24], [45], [10], [-90], [26], [94], [-37], [-4], [16], [21], [-28], [-25], [84], [-10], [61], [-50], [94], [20], [-79], [91], [-48], [0], [-9], [-38], [102], [78], [8], [-8], [69], [24], [57], [36], [5], [-81], [90], [-19], [78], [40], [99], [-30], [46], [-106], [-6], [72], [1], [38], [67], [-16], [-39], [80], [63]]
